@@ -43,11 +43,14 @@ struct glirc_command {
         struct glirc_string command;
 };
 
-typedef void *start_type         (struct glirc *G, const char *path);
-typedef void stop_type           (struct glirc *G, void *S);
-typedef enum process_result process_message_type(struct glirc *G, void *S, const struct glirc_message *);
-typedef enum process_result process_chat_type(struct glirc *G, void *S, const struct glirc_chat *);
-typedef void process_command_type(struct glirc *G, void *S, const struct glirc_command *);
+typedef void *start_type         (struct glirc *G, const char *path, const struct glirc_string *args, size_t args_len);
+typedef void stop_type           (void *S);
+typedef enum process_result process_message_type(void *S, const struct glirc_message *);
+typedef enum process_result process_chat_type(void *S, const struct glirc_chat *);
+typedef void process_command_type(void *S, const struct glirc_command *);
+
+typedef long timer_id;
+typedef void timer_callback(void *dat, timer_id);
 
 struct glirc_extension {
         const char *name;
@@ -67,17 +70,25 @@ int glirc_inject_chat(struct glirc *G,
                 const char* tgt, size_t tgtLen,
                 const char* msg, size_t msgLen);
 char ** glirc_list_networks(struct glirc *G);
-char ** glirc_list_channels(struct glirc *G, struct glirc_string network);
-char ** glirc_list_channel_users(struct glirc *G, struct glirc_string network, struct glirc_string channel);
+char ** glirc_list_channels(struct glirc *G, const char *net, size_t netlen);
+char ** glirc_list_channel_users(struct glirc *G, const char *net, size_t net_len, const char *chan, size_t chan_len);
 void glirc_current_focus(struct glirc *G, char **net, size_t *netlen, char **tgt , size_t *tgtlen);
+void glirc_set_focus(struct glirc *G, const char *net, size_t netlen, const char *tgt , size_t tgtlen);
 char * glirc_my_nick(struct glirc *G, const char *net, size_t netlen);
-void glirc_mark_seen(struct glirc *G, struct glirc_string network, struct glirc_string channel);
-void glirc_clear_window(struct glirc *G, struct glirc_string network, struct glirc_string channel);
-int glirc_identifier_cmp(struct glirc_string s, struct glirc_string t);
+char * glirc_user_account(struct glirc *G, const char *net, size_t netlen, const char *nick, size_t nicklen);
+char * glirc_user_channel_modes(struct glirc *G, const char *net, size_t netlen, const char *chan, size_t chanlen, const char *nick, size_t nicklen);
+char ** glirc_channel_modes(struct glirc *G, const char *net, size_t netlen, const char *chan, size_t chanlen);
+char ** glirc_channel_masks(struct glirc *G, const char *net, size_t netlen, const char *chan, size_t chanlen, char mode);
+void glirc_mark_seen(struct glirc *G, const char *net, size_t net_len, const char *chan, size_t chan_len);
+void glirc_clear_window(struct glirc *G, const char *net, size_t net_len, const char *chan, size_t chan_len);
+int glirc_identifier_cmp(const char *s, size_t s_len, const char *t, size_t t_len);
 int glirc_is_channel(struct glirc *G, const char *net, size_t netlen,
                                       const char *tgt, size_t tgtlen);
 int glirc_is_logged_on(struct glirc *G, const char *net, size_t netlen,
                                         const char *tgt, size_t tgtlen);
+char * glirc_resolve_path(struct glirc *G, const char *path, size_t path_len);
+timer_id glirc_set_timer(struct glirc *G, unsigned long millis, timer_callback *cb, void *dat);
+void *glirc_cancel_timer(struct glirc *G, timer_id tid);
 
 void glirc_free_string(char *);
 void glirc_free_strings(char **);
