@@ -25,7 +25,6 @@ import Control.Monad
 import Control.Applicative hiding (empty)
 
 import           Data.HashMap.Strict (lookup,insertWith,HashMap,empty,unionWith,fromList,toList)
-import           Data.Semigroup
 import           Data.Text (Text)
 import qualified Data.Text as Text
 import           Data.Maybe
@@ -40,7 +39,6 @@ data Recognizer a
 
 instance Monoid (Recognizer a) where
   mempty = Branch "" Nothing empty
-  mappend = both
 
 instance Semigroup (Recognizer a) where
   (<>) = both
