@@ -232,7 +232,6 @@ loadConfiguration ::
   IO (Either ConfigurationFailure (FilePath, Configuration))
 loadConfiguration mbPath = try $
   do (path,txt) <- readConfigurationFile mbPath
-     def  <- loadDefaultServerSettings
      home <- getHomeDirectory
 
      rawcfg <-
@@ -245,7 +244,7 @@ loadConfiguration mbPath = try $
                $ ConfigurationMalformed path
                $ displayException e
        Right cfg ->
-         do cfg' <- resolvePaths path (cfg def home)
+         do cfg' <- resolvePaths path (cfg defaultServerSettings home)
                     >>= validateDirectories path
             return (path, cfg')
 
@@ -257,7 +256,7 @@ resolvePaths file cfg =
      let resolveServerFilePaths = over (ssTlsClientCert . mapped) res
                                 . over (ssTlsClientKey  . mapped) res
                                 . over (ssTlsServerCert . mapped) res
-                                . over (ssSaslEcdsaFile . mapped) res
+                                . over (ssSaslMechanism . mapped . _SaslEcdsa . _3) res
                                 . over (ssLogDir        . mapped) res
      return $! over (configExtensions . mapped . extensionPath) res
              . over (configServers    . mapped) resolveServerFilePaths
