@@ -25,14 +25,13 @@ module Client.Configuration.ServerSettings
   , ssUser
   , ssReal
   , ssPassword
-  , ssSaslUsername
-  , ssSaslPassword
-  , ssSaslEcdsaFile
+  , ssSaslMechanism
   , ssHostName
   , ssPort
   , ssTls
   , ssTlsClientCert
   , ssTlsClientKey
+  , ssTlsClientKeyPassword
   , ssTlsServerCert
   , ssTlsCiphers
   , ssConnectCmds
@@ -48,15 +47,26 @@ module Client.Configuration.ServerSettings
   , ssAutoconnect
   , ssNickCompletion
   , ssLogDir
-  , ssProtocolFamily
+  , ssBindHostName
   , ssSts
   , ssTlsPubkeyFingerprint
   , ssTlsCertFingerprint
   , ssShowAccounts
   , ssCapabilities
 
-  -- * Load function
-  , loadDefaultServerSettings
+  -- * SASL Mechanisms
+  , SaslMechanism(..)
+  , _SaslExternal
+  , _SaslEcdsa
+  , _SaslPlain
+
+  -- * Secrets
+  , Secret(..)
+  , SecretException(..)
+  , loadSecrets
+
+  -- * Defaults
+  , defaultServerSettings
 
   -- * TLS settings
   , UseTls(..)
@@ -71,20 +81,23 @@ import           Client.Commands.Interpolation
 import           Client.Commands.WordCompletion
 import           Client.Configuration.Macros (macroCommandSpec)
 import           Config.Schema.Spec
+import           Control.Exception (Exception, displayException, throwIO, try)
 import           Control.Lens
+import           Control.Monad ((>=>))
 import qualified Data.ByteString as B
 import           Data.Functor.Alt                    ((<!>))
 import           Data.List.NonEmpty (NonEmpty((:|)))
 import           Data.ByteString (ByteString)
-import           Data.Maybe (fromMaybe)
 import           Data.Monoid
 import           Data.Text (Text)
+import qualified Data.List.NonEmpty as NonEmpty
 import           Data.List.Split (chunksOf, splitOn)
 import qualified Data.Text as Text
 import           Irc.Identifier (Identifier, mkId)
-import           Network.Socket (HostName, PortNumber, Family(..))
+import           Network.Socket (HostName, PortNumber)
 import           Numeric (readHex)
-import           System.Environment
+import qualified System.Exit as Exit
+import qualified System.Process as Process
 import           Text.Regex.TDFA
 import           Text.Regex.TDFA.Text (compile)
 
@@ -93,15 +106,14 @@ data ServerSettings = ServerSettings
   { _ssNicks            :: !(NonEmpty Text) -- ^ connection nicknames
   , _ssUser             :: !Text -- ^ connection username
   , _ssReal             :: !Text -- ^ connection realname / GECOS
-  , _ssPassword         :: !(Maybe Text) -- ^ server password
-  , _ssSaslUsername     :: !(Maybe Text) -- ^ SASL username
-  , _ssSaslPassword     :: !(Maybe Text) -- ^ SASL plain password
-  , _ssSaslEcdsaFile    :: !(Maybe FilePath) -- ^ SASL ecdsa private key
+  , _ssPassword         :: !(Maybe Secret) -- ^ server password
+  , _ssSaslMechanism    :: !(Maybe SaslMechanism) -- ^ SASL mechanism
   , _ssHostName         :: !HostName -- ^ server hostname
   , _ssPort             :: !(Maybe PortNumber) -- ^ server port
   , _ssTls              :: !UseTls -- ^ use TLS to connect
   , _ssTlsClientCert    :: !(Maybe FilePath) -- ^ path to client TLS certificate
   , _ssTlsClientKey     :: !(Maybe FilePath) -- ^ path to client TLS key
+  , _ssTlsClientKeyPassword :: !(Maybe Secret) -- ^ client key PEM password
   , _ssTlsServerCert    :: !(Maybe FilePath) -- ^ additional CA certificates for validating server
   , _ssTlsCiphers       :: String            -- ^ OpenSSL cipher suite
   , _ssConnectCmds      :: ![[ExpansionChunk]] -- ^ commands to execute upon successful connection
@@ -117,7 +129,7 @@ data ServerSettings = ServerSettings
   , _ssAutoconnect      :: !Bool -- ^ Connect to this network on server startup
   , _ssNickCompletion   :: WordCompletionMode -- ^ Nick completion mode for this server
   , _ssLogDir           :: Maybe FilePath -- ^ Directory to save logs of chat
-  , _ssProtocolFamily   :: Maybe Family -- ^ Protocol family to connect with
+  , _ssBindHostName     :: Maybe HostName -- ^ Local bind host
   , _ssSts              :: !Bool -- ^ Honor STS policies when true
   , _ssTlsPubkeyFingerprint :: !(Maybe Fingerprint) -- ^ optional acceptable public key fingerprint
   , _ssTlsCertFingerprint   :: !(Maybe Fingerprint) -- ^ optional acceptable certificate fingerprint
@@ -126,6 +138,18 @@ data ServerSettings = ServerSettings
   }
   deriving Show
 
+data Secret
+  = SecretText Text    -- ^ Constant text
+  | SecretCommand (NonEmpty Text) -- ^ Command to generate text
+  deriving Show
+
+-- | SASL mechanisms and configuration data.
+data SaslMechanism
+  = SaslPlain    (Maybe Text) Text Secret -- ^ SASL PLAIN RFC4616 - authzid authcid password
+  | SaslEcdsa    (Maybe Text) Text FilePath -- ^ SASL NIST - https://github.com/kaniini/ecdsatool - authzid keypath
+  | SaslExternal (Maybe Text)      -- ^ SASL EXTERNAL RFC4422 - authzid
+  deriving Show
+
 -- | Regular expression matched with original source to help with debugging.
 data KnownRegex = KnownRegex Text Regex
 
@@ -153,28 +177,23 @@ data Fingerprint
   deriving Show
 
 makeLenses ''ServerSettings
+makePrisms ''SaslMechanism
 
--- | Load the defaults for server settings based on the environment
--- variables.
---
--- Environment variables @USER@, @IRCPASSSWORD@, and @SASLPASSWORD@ are used.
-loadDefaultServerSettings :: IO ServerSettings
-loadDefaultServerSettings =
-  do env  <- getEnvironment
-     let username = Text.pack (fromMaybe "guest" (lookup "USER" env))
-     return ServerSettings
-       { _ssNicks         = pure username
-       , _ssUser          = username
-       , _ssReal          = username
-       , _ssPassword      = Text.pack <$> lookup "IRCPASSWORD" env
-       , _ssSaslUsername  = Nothing
-       , _ssSaslPassword  = Text.pack <$> lookup "SASLPASSWORD" env
-       , _ssSaslEcdsaFile = Nothing
+-- | The defaults for server settings.
+defaultServerSettings :: ServerSettings
+defaultServerSettings =
+  ServerSettings
+       { _ssNicks         = pure "guest"
+       , _ssUser          = "username"
+       , _ssReal          = "realname"
+       , _ssPassword      = Nothing
+       , _ssSaslMechanism = Nothing
        , _ssHostName      = ""
        , _ssPort          = Nothing
        , _ssTls           = UseInsecure
        , _ssTlsClientCert = Nothing
        , _ssTlsClientKey  = Nothing
+       , _ssTlsClientKeyPassword = Nothing
        , _ssTlsServerCert = Nothing
        , _ssTlsCiphers    = "HIGH"
        , _ssConnectCmds   = []
@@ -190,7 +209,7 @@ loadDefaultServerSettings =
        , _ssAutoconnect      = False
        , _ssNickCompletion   = defaultNickWordCompleteMode
        , _ssLogDir           = Nothing
-       , _ssProtocolFamily   = Nothing
+       , _ssBindHostName     = Nothing
        , _ssSts              = True
        , _ssTlsPubkeyFingerprint = Nothing
        , _ssTlsCertFingerprint   = Nothing
@@ -238,14 +257,8 @@ serverSpec = sectionsSpec "server-settings" $
       , req "realname" ssReal anySpec
         "\"GECOS\" name sent to server visible in /whois"
 
-      , opt "sasl-username" ssSaslUsername anySpec
-        "Username for SASL authentication to NickServ"
-
-      , opt "sasl-password" ssSaslPassword anySpec
-        "Password for SASL authentication to NickServ"
-
-      , opt "sasl-ecdsa-key" ssSaslEcdsaFile stringSpec
-        "Path to ECDSA key for non-password SASL authentication"
+      , opt "sasl" ssSaslMechanism saslMechanismSpec
+        "SASL settings"
 
       , req "tls" ssTls useTlsSpec
         "Set to `yes` to enable secure connect. Set to `yes-insecure` to disable certificate checking."
@@ -256,6 +269,9 @@ serverSpec = sectionsSpec "server-settings" $
       , opt "tls-client-key" ssTlsClientKey stringSpec
         "Path to TLS client key"
 
+      , opt "tls-client-key-password" ssTlsClientKeyPassword anySpec
+        "Password for decrypting TLS client key PEM file"
+
       , opt "tls-server-cert" ssTlsServerCert stringSpec
         "Path to CA certificate bundle"
 
@@ -298,8 +314,8 @@ serverSpec = sectionsSpec "server-settings" $
       , opt "log-dir" ssLogDir stringSpec
         "Path to log file directory for this server"
 
-      , opt "protocol-family" ssProtocolFamily protocolFamilySpec
-        "IP protocol family to use for this connection"
+      , opt "bind-hostname" ssBindHostName stringSpec
+        "Source address to bind to before connecting"
 
       , req "sts" ssSts yesOrNoSpec
         "Honor server STS policies forcing TLS connections"
@@ -317,6 +333,28 @@ serverSpec = sectionsSpec "server-settings" $
         "Extra capabilities to unconditionally request from the server"
       ]
 
+saslMechanismSpec :: ValueSpec SaslMechanism
+saslMechanismSpec = plain <!> external <!> ecdsa
+  where
+    mech m   = reqSection' "mechanism" (atomSpec m) "Mechanism"
+    authzid  = optSection "authzid" "Authorization identity"
+    username = reqSection "username" "Authentication identity"
+
+    plain =
+      sectionsSpec "sasl-plain" $ SaslPlain <$
+      optSection' "mechanism" (atomSpec "plain") "Mechanism" <*>
+      authzid <*> username <*> reqSection "password" "Password"
+
+    external =
+      sectionsSpec "sasl-external" $ SaslExternal <$ mech "external" <*>
+      authzid
+
+    ecdsa =
+      sectionsSpec "sasl-ecdsa-nist256p-challenge-mech" $
+      SaslEcdsa <$ mech "ecdsa-nist256p-challenge" <*>
+      authzid <*> username <*>
+      reqSection' "private-key" stringSpec "Private key file"
+
 hookSpec :: ValueSpec HookConfig
 hookSpec =
   flip HookConfig [] <$> anySpec <!>
@@ -352,17 +390,9 @@ fingerprintSpec =
       | ':' `elem` str = splitOn ":" str
       | otherwise      = chunksOf 2  str
 
--- | Specification for IP protocol family.
-protocolFamilySpec :: ValueSpec Family
-protocolFamilySpec =
-      AF_INET   <$ atomSpec "inet"
-  <!> AF_INET6  <$ atomSpec "inet6"
-
-
 nicksSpec :: ValueSpec (NonEmpty Text)
 nicksSpec = oneOrNonemptySpec anySpec
 
-
 useTlsSpec :: ValueSpec UseTls
 useTlsSpec =
       UseTls         <$ atomSpec "yes"
@@ -384,3 +414,31 @@ regexSpec = customSpec "regex" anySpec $ \str ->
   case compile defaultCompOpt ExecOption{captureGroups = False} str of
     Left e  -> Left  (Text.pack e)
     Right r -> Right (KnownRegex str r)
+
+instance HasSpec Secret where
+  anySpec = SecretText <$> textSpec <!>
+            SecretCommand <$> sectionsSpec "command" (reqSection "command" "Command and arguments to execute to secret")
+
+data SecretException = SecretException String String
+  deriving Show
+
+instance Exception SecretException
+
+-- | Run the secret commands in a server configuration replacing them with secret text.
+-- Throws 'SecretException'
+loadSecrets :: ServerSettings -> IO ServerSettings
+loadSecrets =
+  traverseOf (ssPassword             . _Just                  ) (loadSecret "server password") >=>
+  traverseOf (ssSaslMechanism        . _Just . _SaslPlain . _3) (loadSecret "SASL password") >=>
+  traverseOf (ssTlsClientKeyPassword . _Just                  ) (loadSecret "TLS key password")
+
+-- | Run a command if found and replace it with the first line of stdout result.
+loadSecret :: String -> Secret -> IO Secret
+loadSecret _ (SecretText txt) = pure (SecretText txt)
+loadSecret label (SecretCommand (cmd NonEmpty.:| args)) =
+  do let u = Text.unpack
+     res <- try (Process.readProcessWithExitCode (u cmd) (map u args) "")
+     case res of
+       Right (Exit.ExitSuccess,out,_) -> pure (SecretText (Text.pack (takeWhile ('\n' /=) out)))
+       Right (Exit.ExitFailure{},_,err) -> throwIO (SecretException label err)
+       Left ioe -> throwIO (SecretException label (displayException (ioe::IOError)))
