@@ -18,14 +18,12 @@ module Client.EventLoop
 
 import           Client.CApi (popTimer)
 import           Client.Commands
-import           Client.Network.Async (recv)
 import           Client.Configuration (configJumpModifier, configKeyMap, configWindowNames, configDownloadDir)
 import           Client.Configuration.ServerSettings
 import           Client.EventLoop.Actions
 import           Client.EventLoop.Errors (exceptionToLines)
 import           Client.EventLoop.Network (clientResponse)
 import           Client.Hook
-import           Client.Hooks
 import           Client.Image
 import           Client.Image.Layout (scrollAmount)
 import           Client.Log
@@ -195,13 +193,13 @@ doNetworkClose ::
   ClientState {- ^ client state -} ->
   IO ClientState
 doNetworkClose networkId time st =
-  do let (cs,st') = removeNetwork networkId st
+  do let (cs,st1) = removeNetwork networkId st
          msg = ClientMessage
                  { _msgTime    = time
                  , _msgNetwork = view csNetwork cs
                  , _msgBody    = NormalBody "connection closed"
                  }
-     return (recordNetworkMessage msg st')
+     return (recordNetworkMessage msg st1)
 
 
 -- | Respond to a network connection closing abnormally.
@@ -215,14 +213,14 @@ doNetworkError networkId time ex st =
   do let (cs,st1) = removeNetwork networkId st
          st2 = foldl' (\acc msg -> recordError time (view csNetwork cs) (Text.pack msg) acc) st1
              $ exceptionToLines ex
-     reconnectLogic ex cs st2
+     reconnectLogicOnFailure ex cs st2
 
-reconnectLogic ::
+reconnectLogicOnFailure ::
   SomeException {- ^ thread failure reason -} ->
   NetworkState  {- ^ failed network        -} ->
   ClientState   {- ^ client state          -} ->
   IO ClientState
-reconnectLogic ex cs st
+reconnectLogicOnFailure ex cs st
 
   | shouldReconnect =
       do (attempts, mbDisconnectTime) <- computeRetryInfo
@@ -243,7 +241,7 @@ reconnectLogic ex cs st
     shouldReconnect =
       case view csPingStatus cs of
         PingConnecting n _ | n == 0 || n > reconnectAttempts          -> False
-        _ | Just ConnectionFailure{}  <-             fromException ex -> True
+        _ | Just ConnectionFailure{}         <-      fromException ex -> True
           | Just HostnameResolutionFailure{} <-      fromException ex -> True
           | Just PingTimeout         <-              fromException ex -> True
           | Just ResourceVanished    <- ioe_type <$> fromException ex -> True
@@ -279,9 +277,7 @@ doNetworkLine networkId time line st =
                  (stateHook, viewHook)
                       = over both applyMessageHooks
                       $ partition (view messageHookStateful)
-                      $ lookups
-                          (view csMessageHooks cs)
-                          messageHooks
+                      $ view csMessageHooks cs
 
              case stateHook (cookIrcMsg raw) of
                Nothing  -> return st1 -- Message ignored
@@ -325,12 +321,6 @@ parseZncTime = parseTimeM True defaultTimeLocale
              $ iso8601DateFormat (Just "%T%Q%Z")
 
 
--- | Returns the list of values that were stored at the given indexes, if
--- a value was stored at that index.
-lookups :: Ixed m => [Index m] -> m -> [IxValue m]
-lookups ks m = mapMaybe (\k -> preview (ix k) m) ks
-
-
 -- | Update the height and width fields of the client state
 updateTerminalSize :: Vty -> ClientState -> IO ClientState
 updateTerminalSize vty st =
@@ -374,7 +364,8 @@ doAction vty action st =
             let upd Nothing = Just $! view clientFocus st
                 upd x       = x
             in return $! Just $! over clientActivityReturn upd out
-        | otherwise = return (Just out)
+        | otherwise = return $! Just
+                     $! set clientActivityReturn Nothing out
 
       changeEditor  f = continue (over clientTextBox f st)
       changeContent f = changeEditor
