@@ -16,9 +16,8 @@ module Client.EventLoop.Errors
 
 import           Control.Exception
 import           Data.List.NonEmpty (NonEmpty(..))
-import           Network.Socks5
 import           OpenSSL.Session
-import           Hookup
+import           Hookup (ConnectionFailure(..))
 
 -- | Compute the message message text to be used for a connection error
 exceptionToLines ::
@@ -45,9 +44,6 @@ exceptionToLines' ex
   | Just (ProtocolError e) <- fromException ex =
      ("TLS protocol error: " ++ e) :| []
 
-  -- socks package errors
-  | Just err <- fromException ex = explainSocksError err :| []
-
   -- IOErrors, typically network package.
   | Just ioe <- fromException ex =
      explainIOError ioe :| []
@@ -61,9 +57,6 @@ explainIOError ioe = "IO error: " ++ displayException ioe
 explainHookupError :: ConnectionFailure -> NonEmpty String
 explainHookupError e =
   case e of
-    HostnameResolutionFailure ioe ->
-      ("Host not resolved: " ++ displayException ioe) :| []
-
     ConnectionFailure exs ->
       "Connect failed" :| map explainIOError exs
 
@@ -73,16 +66,5 @@ explainHookupError e =
     LineTruncated ->
       "IRC message incomplete" :| []
 
-
-explainSocksError :: SocksError -> String
-explainSocksError ex =
-  case ex of
-    SocksErrorGeneralServerFailure       -> "SOCKS: General server failure"
-    SocksErrorConnectionNotAllowedByRule -> "SOCKS: Connection not allowed by rule"
-    SocksErrorNetworkUnreachable         -> "SOCKS: Network unreachable"
-    SocksErrorHostUnreachable            -> "SOCKS: Host unreachable"
-    SocksErrorConnectionRefused          -> "SOCKS: Connection refused"
-    SocksErrorTTLExpired                 -> "SOCKS: TTL Expired"
-    SocksErrorCommandNotSupported        -> "SOCKS: Command not supported"
-    SocksErrorAddrTypeNotSupported       -> "SOCKS: Address type not supported"
-    SocksErrorOther n                    -> "SOCKS: Unknown error " ++ show n
+    -- the remaining messages can be used as is and on one line
+    _ -> displayException e :| []
