@@ -107,7 +107,9 @@ renderContent st myNick nicks macros pal c = (leftLen, wholeImg)
   leftCur = take (view Edit.pos cur) (view Edit.text cur)
 
   -- ["one","two"] "three" --> "two one three"
-  leftLen = length leftCur + length leftImgs + sum (map imageWidth leftImgs)
+  leftLen = length leftImgs -- separators
+          + sum (map imageWidth leftImgs)
+          + imageWidth (parseIrcText' True (Text.pack leftCur))
 
   rndr = renderLine st pal myNick nicks macros
 
