@@ -50,7 +50,9 @@ module Client.State
 
   -- * Client operations
   , withClientState
-  , clientMatcher, Matcher(..), buildMatcher
+  , clientIsFiltered
+  , clientFilter
+  , buildMatcher
   , clientToggleHideMeta
   , clientHighlightsNetwork
   , channelUserList
@@ -124,6 +126,7 @@ import           Client.State.Focus
 import           Client.State.Network
 import           Client.State.Window
 import           Client.State.DCC
+import           ContextFilter
 import           Control.Applicative
 import           Control.Concurrent.MVar
 import           Control.Concurrent.STM
@@ -153,6 +156,7 @@ import           Irc.RawIrcMsg
 import           Irc.UserInfo
 import           LensUtils
 import           RtsStats (Stats)
+import qualified System.Random.MWC as Random
 import           Text.Regex.TDFA
 import           Text.Regex.TDFA.String (compile)
 
@@ -164,7 +168,7 @@ data ClientState = ClientState
   , _clientActivityReturn    :: !(Maybe Focus)      -- ^ focus prior to jumping to activity
   , _clientFocus             :: !Focus              -- ^ currently focused buffer
   , _clientSubfocus          :: !Subfocus           -- ^ current view mode
-  , _clientExtraFocus        :: ![Focus]            -- ^ extra messages windows to view
+  , _clientExtraFocus        :: ![(Focus, Subfocus)]-- ^ extra messages windows to view
 
   , _clientConnections       :: !(HashMap Text NetworkState) -- ^ state of active connections
   , _clientEvents            :: !(TQueue NetworkEvent)    -- ^ incoming network event queue
@@ -337,8 +341,8 @@ recordChannelMessage network channel msg st
 
     accounts =
       if view (csSettings . ssShowAccounts) cs
-      then view csUsers cs
-      else HashMap.empty
+      then Just (view csUsers cs)
+      else Nothing
 
 
 recordLogLine ::
@@ -592,15 +596,14 @@ clientTick = set clientBell False
 
 -- | Mark the messages on the current window (and any splits) as seen.
 markSeen :: ClientState -> ClientState
-markSeen st =
-  case view clientSubfocus st of
-    FocusMessages -> foldl' aux st focuses
-    _             -> st
+markSeen st = foldl' aux st messageFocuses
   where
     aux acc focus = overStrict (clientWindows . ix focus) windowSeen acc
 
-    focuses = view clientFocus st
-            : view clientExtraFocus st
+    messageFocuses = [focus | (focus, FocusMessages) <- allFocuses]
+
+    allFocuses = (view clientFocus st, view clientSubfocus st)
+               : view clientExtraFocus st
 
 -- | Add the textbox input to the edit history and clear the textbox.
 consumeInput :: ClientState -> ClientState
@@ -650,31 +653,63 @@ clientMatcher st =
            Nothing -> Nothing
            Just r  -> Just r
 
-buildMatcher :: String -> Maybe Matcher
-buildMatcher = go (True, 0, 0)
-  where
-    go (sensitive, b, a) reStr =
-      case dropWhile (' '==) reStr of
-        '-' : 'i' : ' ' : reStr'                                            -> go (False, b, a) reStr'
-        '-' : 'A' : reStr' | [(a' , ' ':reStr'')] <- reads reStr', a'  >= 0 -> go (sensitive, b, a') reStr''
-        '-' : 'B' : reStr' | [(b' , ' ':reStr'')] <- reads reStr', b'  >= 0 -> go (sensitive, b', a) reStr''
-        '-' : 'C' : reStr' | [(num, ' ':reStr'')] <- reads reStr', num >= 0 -> go (sensitive, num, num) reStr''
-        '-' : '-' : reStr' -> finish (sensitive, b, a) (drop 1 reStr')
-        _ -> finish (sensitive, b, a) reStr
+clientIsFiltered :: ClientState -> Bool
+clientIsFiltered = isJust . clientMatcher
 
-    finish (sensitive, b, a) reStr =
-      case compile defaultCompOpt{caseSensitive=sensitive}
+clientFilter :: ClientState -> (a -> LText.Text) -> [a] -> [a]
+clientFilter st f xs =
+  case clientMatcher st of
+    Nothing -> xs
+    Just m  ->
+      filterContext
+        (matcherBefore m)
+        (matcherAfter m)
+        (matcherPred m . f)
+        xs
+
+data MatcherArgs = MatcherArgs
+  { argAfter     :: !Int
+  , argBefore    :: !Int
+  , argInvert    :: !Bool
+  , argSensitive :: !Bool
+  }
+
+defaultMatcherArgs :: MatcherArgs
+defaultMatcherArgs = MatcherArgs
+  { argAfter     = 0
+  , argBefore    = 0
+  , argInvert    = False
+  , argSensitive = True
+  }
+
+buildMatcher :: String -> Maybe Matcher
+buildMatcher = go defaultMatcherArgs
+  where
+    go !args reStr =
+      case dropWhile (' '==) reStr of
+        '-' : 'i' : ' ' : reStr' -> go args{argSensitive=False} reStr'
+        '-' : 'v' : ' ' : reStr' -> go args{argInvert=True} reStr'
+        '-' : 'A' : reStr' | [(a,' ':reStr'')] <- reads reStr', a>=0 -> go args{argAfter=a} reStr''
+        '-' : 'B' : reStr' | [(b,' ':reStr'')] <- reads reStr', b>=0 -> go args{argBefore=b} reStr''
+        '-' : 'C' : reStr' | [(c,' ':reStr'')] <- reads reStr', c>=0 -> go args{argAfter=c,argBefore=c} reStr''
+        '-' : '-' : ' ' : reStr' -> finish args reStr'
+        _ -> finish args reStr
+
+    finish args reStr =
+      case compile defaultCompOpt{caseSensitive=argSensitive args}
                    defaultExecOpt{captureGroups=False}
                    reStr of
         Left{}  -> Nothing
-        Right r -> Just (Matcher b a (matchTest r . LText.unpack))
+        Right r
+          | argInvert args -> Just (Matcher (argBefore args) (argAfter args) (not . matchTest r . LText.unpack))
+          | otherwise      -> Just (Matcher (argBefore args) (argAfter args) (      matchTest r . LText.unpack))
 
 -- | Compute the command and arguments currently in the textbox.
 clientActiveCommand ::
   ClientState           {- ^ client state                     -} ->
   Maybe (String,String) {- ^ command name and argument string -}
 clientActiveCommand st =
-  case break (==' ') (clientFirstLine st) of
+  case break (==' ') (dropWhile (' '==) (clientFirstLine st)) of
     ('/':cmd,_:args) -> Just (cmd,args)
     _                -> Nothing
 
@@ -729,10 +764,31 @@ addConnection attempts lastTime stsUpgrade network st =
                      , _ssHostName = Text.unpack network
                      }
 
-         settings = fromMaybe defSettings
-                  $ preview (clientConfig . configServers . ix network) st
+     eSettings0 <-
+       try $
+       loadSecrets $
+       fromMaybe defSettings $
+       preview (clientConfig . configServers . ix network) st
 
-     now <- getCurrentTime
+     case eSettings0 of
+       Left (SecretException label err) ->
+         do now <- getZonedTime
+            let txt = "Failed loading secret \x02" <> label <> "\x02: " <> err
+            pure $! recordError now network (Text.pack txt) st
+
+       Right settings0 ->
+         do settings1 <- applyStsPolicy stsUpgrade settings0 st
+            -- don't bother delaying on the first reconnect
+            let delay = 15 * max 0 (attempts - 1)
+            c <- createConnection delay settings1
+            seed <- Random.withSystemRandom (Random.asGenIO Random.save)
+            let cs = newNetworkState network settings1 c (PingConnecting attempts lastTime) seed
+            traverse_ (sendMsg cs) (initialMessages cs)
+            pure (set (clientConnections . at network) (Just cs) st)
+
+applyStsPolicy :: Maybe Int -> ServerSettings -> ClientState -> IO ServerSettings
+applyStsPolicy stsUpgrade settings st =
+  do now <- getCurrentTime
      let stsUpgrade'
            | Just{} <- stsUpgrade = stsUpgrade
            | UseInsecure <- view ssTls settings
@@ -741,26 +797,10 @@ addConnection attempts lastTime stsUpgrade network st =
            , now < view stsExpiration policy
            = Just (view stsPort policy)
            | otherwise = Nothing
-
-         settings1 =
-           case stsUpgrade' of
-             Just port -> set ssPort (Just (fromIntegral port))
-                        $ set ssTls UseTls settings
-             Nothing   -> settings
-
-
-         -- don't bother delaying on the first reconnect
-         delay = 15 * max 0 (attempts - 1)
-
-     c <- createConnection
-            delay
-            settings1
-
-     let cs = newNetworkState network settings1 c (PingConnecting attempts lastTime)
-     traverse_ (sendMsg cs) (initialMessages cs)
-
-     return $ set (clientConnections . at network) (Just cs) st
-
+     pure $ case stsUpgrade' of
+              Just port -> set ssPort (Just (fromIntegral port))
+                         $ set ssTls UseTls settings
+              Nothing   -> settings
 
 applyMessageToClientState ::
   ZonedTime                  {- ^ timestamp                -} ->
@@ -832,12 +872,11 @@ scrollClient amt = over clientScroll $ \n -> max 0 (n + amt)
 
 
 -- | List of extra focuses to display as split windows
-clientExtraFocuses :: ClientState -> [Focus]
+clientExtraFocuses :: ClientState -> [(Focus, Subfocus)]
 clientExtraFocuses st =
-  case view clientSubfocus st of
-    FocusMessages -> view clientFocus st `delete` view clientExtraFocus st
-    _             -> []
-
+  delete
+    (view clientFocus st, view clientSubfocus st)
+    (view clientExtraFocus st)
 
 ------------------------------------------------------------------------
 -- Focus Management
@@ -899,21 +938,26 @@ changeFocus focus st
 
     -- Don't deactivate a window if it's going to stay active
     deactivatePrevious
-      | oldFocus `elem` focus : view clientExtraFocus st = id
+      | (oldFocus, FocusMessages) `elem` (focus, FocusMessages) : view clientExtraFocus st = id
       | otherwise = over (clientWindows . ix oldFocus) windowDeactivate
 
 
 -- | Unified logic for assigning to the extra focuses field that activates
 -- and deactivates windows as needed.
-setExtraFocus :: [Focus] -> ClientState -> ClientState
+setExtraFocus :: [(Focus, Subfocus)] -> ClientState -> ClientState
 setExtraFocus newFocuses st
   = aux windowDeactivate newlyInactive
   $ aux windowActivate   newlyActive
   $ set clientExtraFocus newFocuses st
   where
-    newlyActive = newFocuses \\ (view clientFocus st : view clientExtraFocus st)
+    messagePart x = [focus | (focus, FocusMessages) <- x]
 
-    newlyInactive = view clientExtraFocus st \\ (view clientFocus st : newFocuses)
+    current = (view clientFocus st, view clientSubfocus st)
+
+    newlyActive = messagePart newFocuses \\ messagePart (current : view clientExtraFocus st)
+
+    newlyInactive = messagePart (view clientExtraFocus st)
+                 \\ messagePart (current : newFocuses)
 
     aux f xs st1 =
       foldl' (\acc w -> overStrict (clientWindows . ix w) f acc) st1 xs
