@@ -46,6 +46,7 @@ module Client.State.Network
   , csCertificate
   , csMessageHooks
   , csAuthenticationState
+  , csSeed
 
   -- * Cross-message state
   , Transaction(..)
@@ -81,6 +82,7 @@ import           Client.UserHost
 import           Client.Hook (MessageHook)
 import           Client.Hooks (messageHooks)
 import           Control.Lens
+import qualified Control.Monad.ST as ST
 import           Data.HashMap.Strict (HashMap)
 import qualified Data.HashMap.Strict as HashMap
 import qualified Data.HashSet as HashSet
@@ -105,6 +107,7 @@ import           Irc.Modes
 import           Irc.RawIrcMsg
 import           Irc.UserInfo
 import           LensUtils
+import qualified System.Random.MWC as Random
 
 -- | State tracked for each IRC connection
 data NetworkState = NetworkState
@@ -131,6 +134,9 @@ data NetworkState = NetworkState
   , _csPingStatus   :: !PingStatus      -- ^ state of ping timer
   , _csLastReceived :: !(Maybe UTCTime) -- ^ time of last message received
   , _csCertificate  :: ![Text]
+
+  -- Randomization
+  , _csSeed         :: Random.Seed
   }
 
 -- | State of the authentication transaction
@@ -161,7 +167,6 @@ data Transaction
   | NamesTransaction [Text]
   | BanTransaction [(Text,MaskListEntry)]
   | WhoTransaction [UserInfo]
-  | CapTransaction
   | CapLsTransaction [(Text, Maybe Text)]
   deriving Show
 
@@ -242,15 +247,16 @@ newNetworkState ::
   ServerSettings    {- ^ server settings           -} ->
   NetworkConnection {- ^ active network connection -} ->
   PingStatus        {- ^ initial ping status       -} ->
+  Random.Seed       {- ^ initial random seed       -} ->
   NetworkState      {- ^ new network state         -}
-newNetworkState network settings sock ping = NetworkState
+newNetworkState network settings sock ping seed = NetworkState
   { _csUserInfo     = UserInfo "*" "" ""
   , _csChannels     = HashMap.empty
   , _csSocket       = sock
   , _csChannelTypes = defaultChannelTypes
   , _csModeTypes    = defaultModeTypes
   , _csUmodeTypes   = defaultUmodeTypes
-  , _csTransaction  = CapTransaction
+  , _csTransaction  = NoTransaction
   , _csModes        = ""
   , _csSnomask      = ""
   , _csStatusMsg    = ""
@@ -265,6 +271,7 @@ newNetworkState network settings sock ping = NetworkState
   , _csNextPingTime = Nothing
   , _csLastReceived = Nothing
   , _csCertificate  = []
+  , _csSeed         = seed
   }
 
 buildMessageHooks :: [HookConfig] -> [MessageHook]
@@ -294,7 +301,7 @@ applyMessage' msgWhen msg cs =
   case msg of
     Ping args -> ([ircPong args], cs)
     Pong _    -> noReply $ doPong msgWhen cs
-    Join user chan acct ->
+    Join user chan acct _ ->
          ( reply
          , recordUser user acct
          $ overChannel chan (joinChannel (userNick user))
@@ -331,16 +338,22 @@ applyMessage' msgWhen msg cs =
          $ overChannels (nickChange (userNick oldNick) newNick) cs
 
     Reply RPL_WELCOME (me:_) -> doWelcome msgWhen (mkId me) cs
-    Reply RPL_SASLSUCCESS _ -> endCapTransaction cs
-    Reply RPL_SASLFAIL _ -> endCapTransaction cs
+    Reply RPL_SASLSUCCESS _ -> ([ircCapEnd], cs)
+    Reply RPL_SASLFAIL _ -> ([ircCapEnd], cs)
+
     Reply ERR_NICKNAMEINUSE (_:badnick:_)
       | PingConnecting{} <- view csPingStatus cs -> doBadNick badnick cs
+    Reply ERR_BANNEDNICK (_:badnick:_)
+      | PingConnecting{} <- view csPingStatus cs -> doBadNick badnick cs
+    Reply ERR_ERRONEUSNICKNAME (_:badnick:_)
+      | PingConnecting{} <- view csPingStatus cs -> doBadNick badnick cs
+
     Reply RPL_HOSTHIDDEN (_:host:_) ->
         noReply (set (csUserInfo . uiHost) host cs)
 
     -- /who <#channel> %tuhna,616
     Reply RPL_WHOSPCRPL [_me,"616",user,host,nick,acct] ->
-       let acct' = if acct == "0" then "" else acct
+       let acct' = if acct == "0" then "*" else acct
        in noReply (recordUser (UserInfo (mkId nick) user host) acct' cs)
 
     Reply code args        -> noReply (doRpl code msgWhen args cs)
@@ -377,7 +390,6 @@ doWelcome msgWhen me
   . set csNick me
   . set csNextPingTime (Just $! addUTCTime 30 (zonedTimeToUTC msgWhen))
   . set csPingStatus PingNone
-  . set csTransaction NoTransaction -- wipe out CapTransaction if it was active
 
 -- | Handle 'ERR_NICKNAMEINUSE' errors when connecting.
 doBadNick ::
@@ -387,7 +399,22 @@ doBadNick ::
 doBadNick badNick cs =
   case NonEmpty.dropWhile (badNick/=) (view (csSettings . ssNicks) cs) of
     _:next:_ -> ([ircNick next], cs)
-    _        -> ([], cs)
+    _        -> doRandomNick cs
+
+-- | Pick a random nickname now that we've run out of choices
+doRandomNick :: NetworkState -> ([RawIrcMsg], NetworkState)
+doRandomNick cs = ([ircNick candidate], cs')
+  where
+    limit       = 9 -- RFC 2812 puts the maximum nickname length as low as 9!
+    range       = (0, 99999::Int) -- up to 5 random digits
+    suffix      = show n
+    primaryNick = NonEmpty.head (view (csSettings . ssNicks) cs)
+    candidate   = Text.take (limit-length suffix) primaryNick <> Text.pack suffix
+    cs'         = set csSeed seed' cs
+
+    (n, seed')  = ST.runST (do gen <- Random.restore (view csSeed cs)
+                               (,) <$> Random.uniformR range gen <*> Random.save gen
+                           )
 
 doTopic :: ZonedTime -> UserInfo -> Identifier -> Text -> NetworkState -> NetworkState
 doTopic when user chan topic =
@@ -709,31 +736,30 @@ selectCaps cs offered = (supported `intersect` Map.keys capMap)
       | otherwise                                    = []
 
     ss = view csSettings cs
-    sasl = ["sasl" | isJust (view ssSaslUsername ss)
-                   , isJust (view ssSaslPassword ss) ||
-                     isJust (view ssSaslEcdsaFile ss) ||
-                     isJust (view ssTlsClientCert ss) ]
+    sasl = ["sasl" | isJust (view ssSaslMechanism ss) ]
 
 doAuthenticate :: Text -> NetworkState -> ([RawIrcMsg], NetworkState)
 doAuthenticate param cs =
   case view csAuthenticationState cs of
     AS_PlainStarted
-      | "+"       <- param
-      , Just user <- view ssSaslUsername ss
-      , Just pass <- view ssSaslPassword ss
-      -> (ircAuthenticates (encodePlainAuthentication user pass),
+      | "+" <- param
+      , Just (SaslPlain mbAuthz authc (SecretText pass)) <- view ssSaslMechanism ss
+      , let authz = fromMaybe "" mbAuthz
+      -> (ircAuthenticates (encodePlainAuthentication authz authc pass),
           set csAuthenticationState AS_None cs)
 
     AS_ExternalStarted
-      | "+"       <- param
-      , Just user <- view ssSaslUsername ss
-      -> (ircAuthenticates (encodeExternalAuthentication user),
+      | "+" <- param
+      , Just (SaslExternal mbAuthz) <- view ssSaslMechanism ss
+      , let authz = fromMaybe "" mbAuthz
+      -> (ircAuthenticates (encodeExternalAuthentication authz),
           set csAuthenticationState AS_None cs)
 
     AS_EcdsaStarted
-      | "+"       <- param
-      , Just user <- view ssSaslUsername ss
-      -> (ircAuthenticates (Ecdsa.encodeUsername user),
+      | "+" <- param
+      , Just (SaslEcdsa mbAuthz authc _) <- view ssSaslMechanism ss
+      , let authz = fromMaybe authc mbAuthz
+      -> (ircAuthenticates (Ecdsa.encodeAuthentication authz authc),
           set csAuthenticationState AS_EcdsaWaitChallenge cs)
 
     AS_EcdsaWaitChallenge -> ([], cs) -- handled in Client.EventLoop!
@@ -753,10 +779,11 @@ doCap cmd cs =
         prevCaps = view (csTransaction . _CapLsTransaction) cs
 
     CapLs CapDone caps
-      | null reqCaps -> endCapTransaction cs
-      | otherwise    -> ([ircCapReq reqCaps], cs)
+      | null reqCaps -> ([ircCapEnd], cs')
+      | otherwise    -> ([ircCapReq reqCaps], cs')
       where
         reqCaps = selectCaps cs (caps ++ view (csTransaction . _CapLsTransaction) cs)
+        cs' = set csTransaction NoTransaction cs
 
     CapNew caps
       | null reqCaps -> ([], cs)
@@ -767,32 +794,26 @@ doCap cmd cs =
     CapDel _ -> ([],cs)
 
     CapAck caps
-      | "sasl" `elem` caps && isJust (view ssSaslUsername ss) ->
-          if isJust (view ssSaslEcdsaFile ss)
-            then ( [ircAuthenticate Ecdsa.authenticationMode]
-                 , set csAuthenticationState AS_EcdsaStarted cs)
-          else if isJust (view ssSaslPassword ss)
-            then ( [ircAuthenticate "PLAIN"]
-                 , set csAuthenticationState AS_PlainStarted cs)
-          else if isJust (view ssTlsClientCert ss)
-            then ( [ircAuthenticate "EXTERNAL"]
-                 , set csAuthenticationState AS_ExternalStarted cs)
-          else endCapTransaction cs
-      where
-        ss = view csSettings cs
+      | let ss = view csSettings cs
+      , "sasl" `elem` caps
+      , Just mech <- view ssSaslMechanism ss ->
+        case mech of
+          SaslEcdsa{} ->
+            ([ircAuthenticate Ecdsa.authenticationMode],
+             set csAuthenticationState AS_EcdsaStarted cs)
+          SaslPlain{} ->
+            ([ircAuthenticate "PLAIN"],
+             set csAuthenticationState AS_PlainStarted cs)
+          SaslExternal{} ->
+            ([ircAuthenticate "EXTERNAL"],
+             set csAuthenticationState AS_ExternalStarted cs)
 
-    _ -> endCapTransaction cs
-
-endCapTransaction :: NetworkState -> ([RawIrcMsg], NetworkState)
-endCapTransaction cs =
-  case view csTransaction cs of
-    CapTransaction -> ([ircCapEnd], set csTransaction NoTransaction cs)
-    _              -> ([], cs)
+    _ -> ([ircCapEnd], cs)
 
 initialMessages :: NetworkState -> [RawIrcMsg]
 initialMessages cs
    = [ ircCapLs ]
-  ++ [ ircPass pass | Just pass <- [view ssPassword ss]]
+  ++ [ ircPass pass | Just (SecretText pass) <- [view ssPassword ss]]
   ++ [ ircNick (views ssNicks NonEmpty.head ss)
      , ircUser (view ssUser ss) (view ssReal ss)
      ]
