@@ -19,7 +19,6 @@ import           Graphics.Vty.Attributes
 import           Irc.Identifier
 import           Data.HashSet (HashSet)
 import           Data.Foldable
-import           Data.Semigroup
 import           Control.Lens
 
 -- | Render the lines used in a channel mask list.
@@ -29,8 +28,7 @@ ignoreListLines ::
   [Image']
 ignoreListLines ignores pal =
   summaryLine ignores pal :
-  [ text' defAttr (cleanText (idText mask))
-  | mask <- toList ignores ]
+  [ text' defAttr (cleanText (idText mask)) | mask <- toList ignores ]
 
 
 -- | Render a summary describing the number of ignore masks.
